#
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class CallDeal(FormatDeal):
    """
    函数调用call:
        {
            id:id
            type:call
            call|method: import路径+"."+方法名
            vars: [...]
            args: [item_conf, ...]
            maps: {
                key1:item_conf,
                ...
            }
        }
    简写:
        [[id, call], method, args, maps]
        [call, method]
    例:
        [call, buildz.ioc.demo.test.test] //调用buildz.ioc.demo.test下的test方法
    """
    def init(self, fp_lists = None, fp_defaults = None):
        self.singles = {}
        self.sources = {}
        super().init("CallDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "call_lists.js"),
            join(dp, "conf", "call_defaults.js"))
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        conf = edata.conf
        data = self.format(data)
        src = edata.src
        method = xf.g1(data, call=0, method=0)
        #method = xf.g(data, method=0)
        method = self.get_obj(method, conf, src)
        if type(method) in [str, bytes]:
            method = pyz.load(method)
        info = edata.info
        iargs, imaps = None, None
        ivars = None
        if type(info) == dict:
            iargs, imaps,ivars = xf.g(info, args = None, maps = None, vars=None)
        args = xf.g(data, args=[])
        maps = xf.g(data, maps ={})
        vars = xf.g(data, vars={})
        vars = {k:self.get_obj(v, conf, src) for k,v in vars.items()}
        if ivars is not None:
            xf.fill(ivars, vars, 1)
        ivars = vars
        if iargs is not None:
            args = iargs
        if imaps is not None:
            xf.fill(imaps, maps, 1)
        # args = [self.get_obj(v, conf, src, info = edata.info) for v in args]
        # maps = {k:self.get_obj(maps[k], conf, src, info = edata.info) for k in maps}
        self.push_vars(conf, ivars)
        args = [self.get_obj(v, conf, src) for v in args]
        maps = {k:self.get_obj(maps[k], conf, src) for k in maps}
        rst = method(*args, **maps)
        self.pop_vars(conf, ivars)
        return rst

pass



#

from ..tools import *
from buildz.ioc import wrap
import json

try:
    import requests as rq
    session = rq.session()
except:
    rq = None
    session = None

pass
class Request(Base):
    def init(self, http_type, use_session, log, upd, cache=None, debug=False):
        self.type = http_type
        self.cache = cache
        self.use_session = use_session
        self.log = log
        self.upd = upd
        self.fc = None
        self.fcs = None
        self.key_data = None
        self.debug = debug
        self.build()
    def build(self):
        fcs = {}
        global rq, session
        if rq is None:
            return
        obj = rq
        if self.use_session:
            obj = session
        fcs['get'] = [obj.get, "params"]
        fcs['post'] = [obj.post, "data"]
        fcs['json'] = [obj.post, "json"]
        fcs['put'] = [obj.put, "json"]
        fcs['delete'] = [obj.delete, 'json']
        self.fcs = fcs
        self.fc = self.fcs[self.type][0]
        self.key_data = self.fcs[self.type][1]
    def get_set(self, data, kd, maps, km):
        v = xf.get(data, kd)
        if v is not None:
            maps[km] = v
    def req(self, data):
        url = xf.g(data, url=None)
        maps = {}
        self.get_set(data, "data", maps, self.key_data)
        self.get_set(data, "cookies", maps, "cookies")
        self.get_set(data, "headers", maps, "headers")
        self.get_set(data, "proxies", maps, "proxies")
        return self.fc(url, **maps)
    def rsp(self, rp, data):
        url = xf.g(data, url=None)
        xf.s(data, status_code = rp.status_code)
        xf.s(data, result_code = rp.status_code)
        if self.debug:
            self.log.debug(f"request url '{url}' return code: {rp.status_code}")
        show_obj = None
        try:
            show_obj = rp.content
            xf.s(data, result_content=rp.content)
            debug_ct = rp.content
            s = xf.decode(rp.content, "utf-8")
            show_obj = s
            if self.debug:
                self.log.debug(f"request url '{url}' return msg:{s}")
            xf.s(data, result_text=s)
        except Exception as exp:
            self.log.warn(f"exp in deal response on '{url}': {exp}")
        try:
            obj = json.loads(s)
            show_obj = xf.dumps(obj,format=1,deepp=1)
            xf.s(data, result=obj)
        except Exception as exp:
            #self.log.warn(f"exp in deal response on '{url}': {exp}")
            pass
        if self.debug:
            self.log.debug(f"request '{url}' response: {show_obj}")
        try:
            xf.s(data, result_cookies=dict(rp.cookies))
        except Exception as exp:
            self.log.warn(f"exp in deal response on '{url}': {exp}")
        try:
            xf.s(data, result_headers = dict(rp.headers))
        except Exception as exp:
            self.log.warn(f"exp in deal response on '{url}': {exp}")
        return True
    def call(self, data, fc=None):
        if self.fc is None:
            self.log.error("install requests to use this(pip install requests)")
            return False
        if self.cache is not None:
            debug = self.cache.get("request.debug")
            if debug is not None:
                self.debug = debug
        if self.upd is not None:
            data = self.upd(data)
        url = xf.g(data, url=None)
        #self.log.debug(f"request.debug: {self.debug}")
        if self.debug:
            self.log.debug(f"try request url '{url}' with type {self.type}")
        try:
            rp = self.req(data)
        except Exception as exp:
            self.log.error(f"error in request '{url}' with method {self.type}: {exp}")
            return False
        self.rsp(rp, data)
        if fc is None:
            return True
        return fc(data)

pass


@wrap.obj(id="request.post")
@wrap.obj_args("post", "env, request.session, false", "ref, log", "ref, cache.modify", "ref, cache", "env, debug, false")
class Post(Request):
    pass

pass

@wrap.obj(id="request.json")
@wrap.obj_args("json", "env, request.session, false", "ref, log", "ref, cache.modify", "ref, cache", "env, debug, false")
class Json(Request):
    pass

pass

@wrap.obj(id="request.get")
@wrap.obj_args("get", "env, request.session, false", "ref, log", "ref, cache.modify", "ref, cache", "env, debug, false")
class Get(Request):
    pass

pass

@wrap.obj(id="request.put")
@wrap.obj_args("put", "env, request.session, false", "ref, log", "ref, cache.modify", "ref, cache", "env, debug, false")
class Put(Request):
    pass

pass

@wrap.obj(id="request.delete")
@wrap.obj_args("delete", "env, request.session, false", "ref, log", "ref, cache.modify", "ref, cache", "env, debug, false")
class Delete(Request):
    pass

pass



#
from ..ioc.base import Base, EncapeData,IOCError
from ..ioc.single import Single
from ..ioc.decorator import decorator
from .base import FormatData,FormatDeal
from buildz import xf, pyz
from buildz import Base as Basez
import os
g_obj_cid = '_buildz_ioc_conf_index'
dp = os.path.dirname(__file__)
join = os.path.join
class ObjectDeal(FormatDeal):
    """
        对象object:
            {
                id: id
                type: object
                source: 导入路径+调用方法/类
                single: 1 //是否单例，默认是，
                        //这里的单例是一个id对应一个实例，
                        //如果两个id用的同一个source，就是同一个source的两个对象
                // 构造函数
                construct:{
                    args: [
                        item_conf,
                        ...
                    ]
                    maps: {
                        key1: item_conf,
                        ...
                    }
                }
                //construct和args+maps，不能同时存在
                args: [
                    item_conf,
                    ...
                ]
                maps: {
                    key1: item_conf,
                    ...
                }
                // sets之前调用方法
                prev_call: item_conf
                // 对象变量设置属性
                sets: [
                    {key: key1, data: item_conf }
                    ...
                ]
                // sets之后调用
                call: item_conf
                // remove前调用
                remove: item_conf
                // remove后调用
                after_remove: item_conf
            }
        简写：
            [[id, object, single], source, args, maps, sets, calls] 
                sets: [[key1, item_conf], ...]
        极简:
            [object, source]
        例:
            [object, buildz.ioc.ioc.conf.Conf] //生成Conf对象
    """
    def init(self, fp_lists = None, fp_defaults = None, fp_cst = None, fp_set = None):
        self.singles = {}
        self.single = Single("single", "id", 1)
        self.sources = {}
        super().init("ObjectDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "obj_lists.js"),
            join(dp, "conf", "obj_defaults.js"))
        if fp_set is None:
            fp_set = join(dp, 'conf', 'obj_set_lists.js')
        if fp_cst is None:
            fp_cst = join(dp, 'conf', 'obj_cst_lists.js')
        self.fmt_set = FormatData(xf.loads(xf.fread(fp_set)))
        self.fmt_cst = FormatData(xf.loads(xf.fread(fp_cst)))
    def get_maps(self, maps, sid, id):
        if id is None:
            return None
        if sid not in maps:
            return None
        maps = maps[sid]
        if id not in maps:
            return None
        return maps[id]
    def set_maps(self, maps, sid, id, obj):
        if sid not in maps:
            maps[sid] = {}
        maps[sid][id] = obj
    def _deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        data = self.format(data)
        info = edata.info
        conf = edata.conf
        confs = edata.confs
        icst = None
        isets = None
        ivars = None
        if type(info) == dict:
            cid = xf.g(info, id=None)
            iargs, imaps = xf.g(info, args = None, maps = None)
            icst = {'args':iargs, 'maps':imaps}
            if iargs is None and imaps is None:
                icst = None
            isets = xf.g(info, sets = None)
            ivars = xf.g(info, vars=None)
        ids = self.single.get_ids(edata)
        id = xf.g(data, id = None)
        #print(f"obj.deal ids: {ids} for {data}")
        obj = self.single.get_by_ids(ids)
        if obj is not None:
            #raise IOCError(f"null for {ids}")
            return obj
        #source = xf.g(data, source=0)
        source = xf.g1(data, source=0, src=0)
        if source == 0:
            raise Exception(f"define object without 'source' key, {data}")
        source = self.get_obj(source, conf)
        if type(source)==str:
            fc = xf.get(self.sources, source, None)
        else:
            fc = source
        if fc is None:
            fc = pyz.load(source)
            self.sources[source]=fc
        cst = xf.g(data, construct = None)
        if cst is None:
            _args = xf.g(data, args = [])
            _maps = xf.g(data, maps = {})
            cst = [_args, _maps]
        cst = self.fmt_cst(cst)
        if icst is not None:
            xf.fill(icst, cst, 1)
        args = xf.g(cst, args=[])
        args = xf.im2l(args)
        maps = xf.g(cst, maps={})
        self.push_vars(conf, ivars)
        args = [self.get_obj(v, conf) for v in args]
        maps = {k:self.get_obj(maps[k], conf) for k in maps}
        obj = self.single.get_by_ids(ids)
        if obj is not None:
            return obj
        obj = fc(*args, **maps)
        self.single.set_by_ids(ids, obj)
        prev_call = xf.g(data, prev_call=None)
        if prev_call is not None:
            # TODO: 这边info透传不知道会不会有问题
            self.get_obj(prev_call, conf, obj, edata.info)
        sets = xf.g(data, sets=[])
        if type(sets)==list:
            tmp = {}
            for kv in sets:
                kv = self.fmt_set(kv)
                k = kv['key']
                v = xf.get_first(kv, "val", "data")
                tmp[k] = v
            sets = tmp
        if type(isets) == list:
            tmp = {}
            for kv in isets:
                kv = self.fmt_set(kv)
                k = kv['key']
                v = xf.get_first(kv, "val", "data")
                tmp[k] = v
            isets = tmp
        if isets is not None:
            xf.fill(isets, sets, 1)
        for k,v in sets.items():
            v = self.get_obj(v, conf, obj)#, edata.info)
            setattr(obj, k, v)
        call = xf.g(data, call=None)
        if call is not None:
            self.get_obj(call, conf, obj)#, edata.info)
        self.pop_vars(conf, ivars)
        #print(f"obj.deal ids: {ids} for {data}, rst: {obj}")
        return obj
    def remove(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        data = self.format(data)
        info = edata.info
        conf = edata.conf
        confs = edata.confs
        ids = self.single.get_ids(edata)
        obj = self.single.get_by_ids(ids)
        if obj is None:
            return None
        call = xf.g(data, remove=None)
        if call is not None:
            self.get_obj(call, conf, obj, edata.info)
        xf.removes(self.singles, ids)
        call = xf.g(data, after_remove=None)
        if call is not None:
            self.get_obj(call, conf, obj, edata.info)
        return None

pass
def update_set(maps):
    rst = {}
    for k,v in maps.items():
        if type(v)==str:
            v = xf.loads(v)
        rst[k] = v
    return rst

pass
def update_list(arr):
    rst = []
    for v in arr:
        if type(v)==str:
            v = xf.loads(v)
        rst.append(v)
    return rst

pass
ioc_conf_key = "_buildz_ioc_conf"
class IOCConf(Basez):
    def init(self, key = "_buildz_ioc_conf", ckey = "_buildz_ioc_conf_cls"):
        self.key = key
        self.ckey = ckey
    def get(self, cls, default):
        if not hasattr(cls, self.ckey):
            return default
        prv = getattr(cls, self.ckey)
        if prv!=cls:
            return default
        if hasattr(cls, self.key):
            return getattr(cls, self.key)
        return default
    def set(self, cls, dt):
        setattr(cls, self.key, dt)
        setattr(cls, self.ckey, cls)

pass
g_ioc_conf = IOCConf()
class IOCObjectAdd_(Basez):
    def init(self, key, *arr):
        _arr = update_list(arr)
        self.key = key
        self._arr = _arr
    def call(self, cls):
        rst = {}
        rst = g_ioc_conf.get(cls, rst)
        _arr = []
        if self.key in rst:
            _arr = rst[self.key]
        _arr+=self._arr
        rst[self.key] = _arr
        g_ioc_conf.set(cls, rst)
        return cls

pass
class IOCObjectArgs(IOCObjectAdd_):
    def init(self, *arr):
        super().init("args", *arr)

pass
class IOCObjectMCall(IOCObjectAdd_):
    def init(self, *arr):
        # function, args, maps
        ks = "mcall,args,maps".split(",")
        _arr = []
        for item in arr:
            if type(item)==str:
                item = xf.loads(item)
            if type(item)==list:
                item = update_list(item)
                l = min(len(ks), len(item))
                _tmp = {}
                for i in range(l):
                    _tmp[ks[i]] = item[i]
                item = _tmp
            else:
                item = update_set(item)
            item['source'] = None
            item['type'] = "mcall"
            _arr.append(item)
        super().init("mcalls", *_arr)

pass
class IOCObjectSet_(Basez):
    def init(self, key, **maps):
        _maps = update_set(maps)
        self._maps = {key:_maps}
    def call(self, cls):
        rst = {}
        rst = g_ioc_conf.get(cls, rst)
        xf.fill(self._maps, rst)
        g_ioc_conf.set(cls, rst)
        return cls

pass
class IOCObjectSet(IOCObjectSet_):
    def init(self, **maps):
        super().init("sets", **maps)

pass
class IOCObjectMap(IOCObjectSet_):
    def init(self, **maps):
        super().init("maps", **maps)

pass
class IOCObject(Basez):
    KEYS = "id,args,maps,call,prev_call,single,remove,sets,after_remove".split(",")
    SET_KEYS = "maps,sets".split(",")
    def init(self, **maps):
        rst = update_set(maps)
        for key in self.SET_KEYS:
            if key not in maps:
                continue
            val = maps[key]
            for _k, _v in val.items():
                if type(_v)==str:
                    _v = xf.loads(_v)
                val[_k] = _v
        self._maps = rst
    def _set(self, key, **maps):
        sets = {}
        if key in self._maps:
            sets = self._maps[key]
        rst = update_set(maps)
        xf.fill(sets, rst)
        self._maps[key] = rst
        return self
    def map(self, **maps):
        return self._set("maps", **maps)
    def maps(self, **maps):
        return self.map(**maps)
    def set(self, **maps):
        return self._set("sets", **maps)
    def sets(self, **maps):
        return self.set(**maps)
    def call(self, cls):
        src = cls.__module__+"."+cls.__name__
        conf = {}
        conf = g_ioc_conf.get(cls, conf)
        xf.fill(self._maps, conf)
        conf['source'] = src
        conf['type'] = 'object'
        if 'mcalls' in conf and 'call' not in conf:
            conf['call'] = {'type': "calls", 'calls': conf['mcalls']}
        conf[g_obj_cid] = decorator.add_datas(conf)
        g_ioc_conf.set(cls, conf)
        return cls

pass

#decorator.regist("IOCObject", IOCObject)
decorator.regist("obj", IOCObject)
decorator.regist("object", IOCObject)
decorator.regist("obj_set", IOCObjectSet)
decorator.regist("obj_map", IOCObjectMap)
decorator.regist("obj_arg", IOCObjectArgs)
decorator.regist("obj_mcall", IOCObjectMCall)
decorator.regist("obj_sets", IOCObjectSet)
decorator.regist("obj_maps", IOCObjectMap)
decorator.regist("obj_args", IOCObjectArgs)


#

from ..tools import *
from buildz.ioc import wrap
@wrap.obj(id="verify")
@wrap.obj_args("ref, log", "ref, cache.modify")
class Verify(Base):
    def init(self, log, upd):
        self.log = log
        self.upd = upd
        self.opts = {}
        self.opts[">"] = self.make_val_cmp(lambda val,v:val>v)
        self.opts["<"] = self.make_val_cmp(lambda val,v:val<v)
        self.opts[">="] = self.make_val_cmp(lambda val,v:val>=v)
        self.opts["<="] = self.make_val_cmp(lambda val,v:val<=v)
        self.opts["="] = self.eq
        self.opts["!="] = self.neq
    def eq(self, val, v):
        if type(val)!=type(v):
            if type(val)==str or type(v)==str:
                return str(val)==str(v)
        return val==v
    def neq(self, val, v):
        if type(val)!=type(v):
            if type(val)==str or type(v)==str:
                return str(val)!=str(v)
        return val!=v
    def make_val_cmp(self, opt):
        def fc(val, v):
            if val is None:
                return False
            if type(val)==str:
                val = float(val)
            if type(v)==str:
                v = float(v)
            return opt(val, v)
        return fc
    def match(self, v, val, result, data):
        tp,v = v
        if tp in self.opts:
            return self.opts[tp](val,v)
        if tp=="eval":
            return eval(v)
        elif tp == "exec":
            exec(v)
            return self.val
        else:
            err = f"not impl match type: {tp}"
            self.log.error(err)
            raise Exception(err)
    def call(self, data, fc=None):
        data = self.upd(data)
        note = xf.g(data, note=data)
        result = xf.g(data, result = {})
        vs = xf.g(data, verify=[])
        for it in vs:
            if type(it)==str:
                it = xf.loads(it)
            k,v=it
            bak = v
            if k == "$":
                if type(v)!=list:
                    v = ["eval", v]
                val = None
            else:
                val = xf.gets(data, k.split("."))
            if type(v)!=list:
                v = ["=", v]
            jg = self.match(v, val, result, data)
            if not jg:
                self.log.error(f"verify failed in {note}, key: '{k}', match: {bak}, val: {val}")
                return False
        if fc is None:
            return True
        return fc(data)

pass




from .loaderz import mg, buffer, base
from . import file
from .loaderz.deal import nextz, spt, strz, listz, spc, setz, mapz, reval, lrval, listmapz
class BoolFc:
    def __init__(self, mg):
        trues = [mg.like(k) for k in ["true", "True", "1"]]   
        trues += [1,True] 
        falses = [mg.like(k) for k in ["false", "False", "0"]]
        falses += [0,False]
        self.trues = set(trues)
        self.falses = set(falses)
    def __call__(self, val):
        if val in self.trues:
            return True
        elif val in self.falses:
            return False
        else:
            raise Exception("unknown bool val")

pass
def build_lrval(mgs):
    bl = BoolFc(mgs)
    fcs = lrval.Fcs()
    fcs.set("float", float)
    fcs.set("f", float)
    fcs.set("int", int)
    fcs.set("i", int)
    fcs.set("bool", bl)
    fcs.set("bl", bl)
    fcs.set("null", lambda x:None)
    fcs.set("nil", lambda x:None)
    fcs.set("n", lambda x:None)
    mgs.add(lrval.LRValDeal("<",">",fcs))

pass
def build_val(mgs):
    mgs.add(reval.ValDeal("[\+\-]?\d+", int))
    mgs.add(reval.ValDeal("[\+\-]?\d*\.\d+", float))
    mgs.add(reval.ValDeal("[\+\-]?\d*(?:\.\d+)?e[\+\-]?\d+", float))
    mgs.add(reval.ValDeal("null", lambda x:None))
    mgs.add(reval.ValDeal("true", lambda x:True))
    mgs.add(reval.ValDeal("false", lambda x:False))

pass
g_mg = None
def build(as_bytes=False):
    global g_mg
    if g_mg is not None and g_mg.as_bytes == as_bytes:
        return g_mg
    mgs = mg.Manager(as_bytes)
    mgs.add(setz.SetDeal(':'))
    mgs.add(setz.SetDeal('='))
    mgs.add(spt.PrevSptDeal(",",1))
    mgs.add(spt.PrevSptDeal(';',1))
    mgs.add(spt.PrevSptDeal('\n'))
    build_lrval(mgs)
    mgs.add(listz.ListDeal("(", ")"))
    mgs.add(listz.ListDeal("[", "]"))
    mgs.add(mapz.MapDeal("{", "}"))
    build_val(mgs)
    #1,0,0,1: 没引号当r"..."
    #1,0,1,1: 没引导当"..."
    mgs.add(strz.PrevStrDeal("r'''","'''",0,0,0))
    mgs.add(strz.PrevStrDeal('r"""','"""',0,0,0))
    mgs.add(strz.PrevStrDeal("r'","'",1,0,0,1))
    mgs.add(strz.PrevStrDeal('r"','"',1,0,0))
    mgs.add(strz.PrevStrDeal("###","###",0,1))
    mgs.add(strz.PrevStrDeal("/*","*/",0,1))
    mgs.add(strz.PrevStrDeal("'''","'''",0,0,1))
    mgs.add(strz.PrevStrDeal('"""','"""',0,0,1))
    mgs.add(strz.PrevStrDeal("#","\n",1,1))
    mgs.add(strz.PrevStrDeal("//","\n",1,1))
    mgs.add(strz.PrevStrDeal("'","'",1,0,1))
    mgs.add(strz.PrevStrDeal('"','"',1,0,1))
    mgs.add(nextz.PrevNextDeal())
    g_mg = mgs
    return mgs

pass
g_mgx = None
def build_args(as_bytes=False):
    global g_mgx
    if g_mgx is not None and g_mgx.as_bytes == as_bytes:
        return g_mgx
    mgs = mg.Manager(as_bytes)
    mgs.add(setz.SetDeal(':'))
    mgs.add(setz.SetDeal('='))
    mgs.add(spt.PrevSptDeal(",",1))
    mgs.add(spt.PrevSptDeal(';',1))
    mgs.add(spt.PrevSptDeal('\n'))
    mgs.add(spt.PrevSptDeal(' '))
    build_lrval(mgs)
    # mgs.add(listz.ListDeal("(", ")"))
    # mgs.add(listz.ListDeal("[", "]"))
    # mgs.add(mapz.MapDeal("{", "}"))
    mgs.add(listmapz.ListMapDeal("(", ")"))
    mgs.add(listmapz.ListMapDeal("[", "]"))
    mgs.add(listmapz.ListMapDeal("{", "}", True))
    build_val(mgs)
    #1,0,0,1: 没引号当r"..."
    #1,0,1,1: 没引导当"..."
    mgs.add(strz.PrevStrDeal("r'''","'''",0,0,0))
    mgs.add(strz.PrevStrDeal('r"""','"""',0,0,0))
    mgs.add(strz.PrevStrDeal("r'","'",1,0,0,1))
    mgs.add(strz.PrevStrDeal('r"','"',1,0,0))
    mgs.add(strz.PrevStrDeal("###","###",0,1))
    mgs.add(strz.PrevStrDeal("/*","*/",0,1))
    mgs.add(strz.PrevStrDeal("'''","'''",0,0,1))
    mgs.add(strz.PrevStrDeal('"""','"""',0,0,1))
    mgs.add(strz.PrevStrDeal("#","\n",1,1))
    mgs.add(strz.PrevStrDeal("//","\n",1,1))
    mgs.add(strz.PrevStrDeal("'","'",1,0,1))
    mgs.add(strz.PrevStrDeal('"','"',1,0,1))
    mgs.add(nextz.PrevNextDeal())
    g_mgx = mgs
    return mgs

pass
def load(read, as_bytes = False):
    mgs = build(as_bytes)
    return msg.loads(read)

pass
def is_args(obj):
    return type(obj)==listmapz.Args

pass
def loads_args(s, as_args = False, as_bytes = False):
    mgs = build_args(type(s)==bytes)
    #input = buffer.BufferInput(s)
    rst = mgs.loads(s)
    if not as_args:
        return rst
    if is_args(rst):
        return rst
    if type(rst) not in [list, dict]:
        rst = [rst]
    if type(rst)==list:
        rst = [rst, {}]
    else:
        rst = [[], rst]
    return listmapz.Args(*rst)

pass
loadx = loads_args
def loads(s):
    mgs = build(type(s)==bytes)
    #input = buffer.BufferInput(s)
    return mgs.loads(s)

pass

def loadf(fp, bts = False):
    if bts:
        s = file.bread(fp)
    else:
        s = file.fread(fp)
    return loads(s)

pass



from .writer import mg, itemz, base, conf
from .writer.deal import listz, mapz, strz, reval, jsonval
from . import file
pts = [
    "[\+\-]?\d+",
    "[\+\-]?\d+\.\d+",
    "[\+\-]?\d+e[\+\-]?\d+",
    "null",
    "true",
    "false",
    "[\s\S]*[\n\r\t\:\[\]\{\}\(\)\,\:\=\'\<\>\" \|\#\;\/][\s\S]*"
]
def build(json_format=False):
    mgs = mg.Manager()
    if not json_format:
        mgs.add(strz.StrDeal('"','"', pts))
        mgs.add(reval.ValDeal(float, lambda x:str(x)))
        mgs.add(reval.ValDeal(int, lambda x:str(x)))
        mgs.add(reval.ValDeal(type(None), lambda x:'null'))
        mgs.add(reval.ValDeal(bool, lambda x:'true' if x else 'false'))
    else:
        mgs.add(jsonval.ValDeal())
    mgs.add(listz.ListDeal('[',']',','))
    mgs.add(mapz.MapDeal('{','}',':',','))
    return mgs

pass
def dumps(obj, bytes = 0, format = 0, deep = 0, json_format= 0):
    cf = conf.Conf()
    cf.set(bytes=bytes, format=format, deep=deep)
    if format:
        cf.set(set=1, prev=1,line=4, spc=' ')
    else:
        cf.set(set=1, prev=1)
    mgs = build(json_format)
    return mgs.dump(obj, cf)

pass

def dumpf(obj, filepath, bytes = 0, format = 0, deep = 0, json_format= 0, mode = 'w'):
    s = dumps(obj, bytes = bytes, format = format, deep = deep, json_format= json_format)
    file.fwrite(s, filepath, mode)

pass

def dump(output, obj, *argv, **maps):
    rs = dumps(obj, *argv, **maps)
    output(rs)

pass



#
from .. import xf
from .. import ioc
from ..base import Base
from ..ioc import wrap

@wrap.obj(id="list")
@wrap.obj_args("[ref, [env, buildz.auto.deal, auto.deal], [ref, autoz.deal]]", "[ioc, confs]")
class List(Base):
    def init(self, deal, mg):
        self.deal = deal
        self.curr_deal = deal
        self.mg = mg
    def curr(self):
        return self.curr_deal
    def call(self, maps, fp):
        datas = xf.g(maps, datas = [])
        sdeal = xf.g(maps, deal = None)
        deal_obj = xf.g(maps, deal_obj = None)
        deal = self.deal
        if deal_obj is not None:
            deal = deal_obj
        elif sdeal is not None:
            deal = self.mg.get(sdeal)
        self.curr_deal = deal
        for data in datas:
            if not deal(data):
                return False
        return True

pass
#wrap.add_datas("[(env, env.buildz.auto.deal), buildz.auto.deal, auto.deal]")
@wrap.obj_args("[env, buildz.auto.deal, auto.deal]")
@wrap.obj(id="autoz.deal")
class DfDeal(Base):
    def init(self, id):
        self.id = id
    def call(self, data):
        print(f"[ERROR] implement obj with id '{self.id}' by yourself")

pass






#


from .. import xf
from .. import ioc
from ..base import Base
from ..ioc import wrap
from ..tools import *
import time, sys
class Log(Base):
    def init(self, shows = None):
        if shows is None:
            shows = ["info", "debug", "warn", "error"]
        self.shows=shows
    def log(self, level, *args):
        raise Exception("unimpl")
    def info(self, *args):
        if "info" not in self.shows:
            return
        self.log("INFO", *args)
    def warn(self, *args):
        if "warn" not in self.shows:
            return
        self.log("WARN", *args)
    def debug(self, *args):
        if "debug" not in self.shows:
            return
        self.log("DEBUG", *args)
    def error(self, *args):
        if "error" not in self.shows:
            return
        self.log("ERROR", *args)

pass
class FpLog(Log):
    def init(self, fp = None,shows =None):
        super().init(shows)
        self.fp = fp
    def log(self, level, *args):
        args = [str(k) for k in args]
        msg = " ".join(args)
        date = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        msg = f"[{level}] {date} {msg}\n"
        sys.stdout.write(msg)
        if self.fp is not None:
            fp = time.strftime(self.fp)
            fz.makefdir(fp)
            fz.write(msg.encode("utf-8"), fp, 'ab')

pass



#
from ..tools import *
from buildz.ioc import wrap
@wrap.obj(id="save")
@wrap.obj_args("ref, cache", "ref, log", "ref, cache.modify")
class Save(Base):
    def init(self, cache, log, upd):
        self.cache = cache
        self.log = log
        self.upd = upd
    def call(self, data, fc=None):
        data = self.upd(data)
        save = xf.g(data, save={})
        for k, v in save.items():
            if type(v)==str:
                v = ["key"]+v.split(".")
            if type(v)!=list:
                v = ["key", v]
            tp = v[0]
            v = v[1:]
            if tp=="eval":
                val = eval(v[0])
            elif tp == "exec":
                exec(v[0])
                val = self.val
            else:
                val = xf.gets(data, v)
            self.cache.set(k, val)
        return True

pass





#
from ..tools import *
from buildz.ioc import wrap
import os
@wrap.obj(id="deal.list")
@wrap.obj_args("ref, cache", "ref, log", "ref, cache.modify", "ref, list")
class List(Base):
    def init(self, cache, log, upd, lst):
        self.cache = cache
        self.log = log
        self.upd = upd
        self.lst = lst
    def call(self, data, fc):
        data = self.upd(data)
        fp = xf.g(data, file = None)
        if fp is not None:
            fp = self.cache.rfp(fp)
        datas = xf.g(data, datas=[])
        if fp is not None and os.path.isfile(fp):
            datas = xf.loadf(fp)
        if type(datas)==dict:
            datas = xf.g(datas, datas=[])
        deal = self.lst.curr()
        print(f"List.datas: {xf.dumps(datas,format=1,deep=1)}")
        for data in datas:
            if not deal(data):
                return False
        return True

pass





from .. import base
from .. import item

class PrevSptDeal(base.BaseDeal):
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def init(self, spt, allow_empty = False):
        self.spt = spt
        self.allow_empty = allow_empty
        self.l = len(spt)
    def prev(self, buffer, queue, pos):
        c = buffer.read(self.l)
        if not self.same(c, self.spt):
            return False
        buffer.pop_read(self.l)
        data = buffer.full()
        buffer.clean()
        crr = pos.get()
        pos.update(data)
        item_spt = item.PrevItem(self.spt, pos.get(), self.id(), src='spt', allow_empty = self.allow_empty)
        pos.update(c)
        data = data.strip()
        if len(data)==0 and not self.allow_empty:
            queue.append(item_spt)
            return True
        if len(data)==0:
            if len(queue)>0 and queue[-1].any(is_val = 1, right=1):
                queue.append(item_spt)
                return True
        obj = item.PrevItem(data, crr, is_val = 1, src='spt')
        queue.append(obj)
        queue.append(item_spt)
        return True
    def deal(self, queue, stack):
        bl = False
        while len(queue)>0 and queue[0].check(type = self.id()):
            queue.pop(0)
            bl = True
        return bl

pass


#coding=utf-8
from buildz.base import Base
from buildz import xf
class Decorator(Base):
    def init(self):
        self.conf = {}
        self.fcs = {}
        self.regist("add_datas", self.add_datas)
    def regist(self, key, fc):
        self.fcs[key] = fc
    def get(self, tag, index):
        if tag not in self.conf:
            self.conf[tag]=[]
        return self.conf[tag][index]
    def add(self, tag, data):
        if tag not in self.conf:
            self.conf[tag]=[]
        id = len(self.conf[tag])
        self.conf[tag].append(data)
        return id
    def set(self, tag, key, val):
        if tag not in self.conf:
            self.conf[tag]={}
        self.conf[tag][key]=val
    def add_datas(self, item):
        if type(item)==str:
            item = xf.loads(item)
        return self.add("datas", item)
    def get_datas(self, id):
        return self.get("datas", id)
    def set_datas(self, id, val):
        return self.set("datas", id, val)
    def set_envs(self, key, val):
        return self.set("env", key, val)
    def add_inits(self, val):
        return self.add("inits", val)
    def add_locals(self, item):
        return self.add("locals", item)
    def call(self):
        return self.conf

pass

decorator = Decorator()



class Conf:
    def none(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return False
        return True
    def any(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return True
        return False
    def check(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)!=v:
                return False
        return True
    def maps(self, key):
        if key not in self._maps:
            if key == 'type':
                return self.type
            return None
        return self._maps[key]
    def set(self, **maps):
        for k in maps:
            v = maps[k]
            self._maps[k] = v
    def get(self, **maps):
        rst = []
        for k in maps:
            df = maps[k]
            v = self.maps(k)
            if v is None:
                v = df
            rst.append(v)
        if len(rst)==1:
            rst = rst[0]
        return rst
    def __init__(self, *argv, **maps):
        self.init(*argv, **maps)
    def init(self, **maps):
        self._maps = maps
    def s(self, val):
        #print("bytes:", self.get(bytes=0))
        if self.get(bytes=0):
            if type(val)==str:
                val = val.encode(self.get(code="utf-8"))
        else:
            if type(val)==bytes:
                val = val.decode(self.get(code="utf-8"))
        return val

pass


#coding=utf-8
from buildz import xf, pyz
from .decorator import decorator
from buildz.xf import g as xg
from buildz import argx
import json
from .base import Base, EncapeData,IOCError, IdNotFoundError
from .conf import Conf
import os
class ConfsNode(Base):
    def init(self):
        self.confs = []
        self.ids = {}

pass


class Confs(Base):
    """

    """
    def flush_env(self, envs):
        """
            a.b.c:d -> a:{b:{c:d}}
        """
        for key in list(envs.keys()):
            val = envs[key]
            if type(val)==dict:
                self.flush_env(val)
            ids = self.env_ids(key)
            if len(ids)>1:
                del envs[key]
            pids = ids[:-1]
            id = ids[-1]
            tmp = envs
            for pid in pids:
                if pid not in tmp:
                    tmp[pid] = {}
                tmp = tmp[pid]
            if id not in tmp:
                tmp[id] = val
                continue
            tval = tmp[id]
            if type(tval) == dict and type(val)==dict:
                self.update_maps(tval, val)
            else:
                tmp[id] = val
    def env_ids(self, id):
        return id.split(self.env_spt)
    def env_id(self, ids):
        return self.env_spt.join(ids)
    def get_env_sys(self, id, sid=None):
        sysdt = os.getenv(id)
        return sysdt
    def build_env_args_xf(self):
        data = xf.args()
        if xf.is_args(data):
            data = data.maps
        env = xf.g(data, env={})
        self.envs_args = env
        self.flush_env(self.envs_args)
    def build_env_args_buildz(self):
        args, maps = argx.fetch()
        env = maps
        # e = xf.get(maps, e = [])
        # env = xf.get(maps, env=[])
        # env += e
        # env = [k.split("=") for k in env]
        # env = {k[0]:"=".join(k[1:]) for k in env}
        self.envs_args = env
        self.flush_env(self.envs_args)
    def build_env_args(self):
        if self.args_type == "xf":
            self.build_env_args_xf()
        else:
            self.build_env_args_buildz()
    def get_env_args(self, id, sid=None):
        if self.envs_args is None:
            self.build_env_args()
        return self.get_env_maps(id, self.envs_args)
    def get_env_local(self, id, sid=None):
        if sid is not None and not self.global_env:
            val = self.confs[sid].get_env(id, False)
            if val is not None:
                return val
        return None
    def get_env_maps(self, id, maps):
        ids = self.env_ids(id)
        #print(f"[TZBZ] get_env_maps({id})[{ids}] in {maps}")
        envs = maps
        for id in ids:
            if type(envs)!=dict:
                envs = None
                break
            if id not in envs:
                envs = None
                break 
            envs = envs[id]
        return envs
    def get_env_conf(self, id, sid=None):
        return self.get_env_maps(id, self.envs)
    def enable_args(self, enable = True):
        key = "args"
        if enable:
            if key in self.env_orders:
                return
            self.env_orders = [key]+self.env_orders
        else:
            if key not in self.env_orders:
                return
            self.env_orders.remove(key)
    def set_args_type(self, args_type):
        self.args_type = args_type
    def get_env(self, id, sid=None):
        for key in self.env_orders:
            fc = self.env_fcs[key]
            obj = fc(id, sid)
            #print(f"[TZBZ] get_env.{key}.{fc}({id}, {sid}): {obj}")
            if obj is not None:
                return obj
        return None
        if sid is not None and not self.global_env:
            val = self.confs[sid].get_env(id, False)
            if val is not None:
                return val
        sysdt = os.getenv(id)
        if sysdt is not None:
            return sysdt
        ids = self.env_ids(id)
        envs = self.envs
        for id in ids:
            if type(envs)!=dict:
                envs = None
                break
            if id not in envs:
                envs = None
                break 
            envs = envs[id]
        return envs
    def set_envs(self, maps):
        self.flush_env(maps)
        self.update_env(maps)
    def set_env(self, id, val):
        obj = {id:val}
        self.flush_env(obj)
        self.update_maps(self.envs, obj)
    def update_env(self, obj):
        self.update_maps(self.envs, obj)
    def set_deal(self, type, fc):
        self.deals[type] = fc
    def init_fp(self, fp):
        conf = self.loads(xf.fread(fp))
        self.init(conf, self.loads)
    def by_json(self):
        self.by('json')
    def by_xf(self):
        self.by('xf')
    def by(self, type = 'xf'):
        if type == 'xf':
            self.loads = xf.loads
        elif type == 'json':
            self.loads = json.loads
        else:
            raise Exception("only 'xf' and 'json' impl now")
    def get_key(self, obj, key = 'id', index=0):
        if type(obj)==dict:
            return obj[key]
        id = obj[index]
        if type(id) in [list, tuple]:
            return id[0]
        return id
    def init(self, conf={}, loads = None):
        """
        {
            // 环境变量的分隔符
            // default = '.'
            env_spt: .
            // 数据id的分隔符
            // default = "."
            spt: .
            // 数据的默认类型(处理方式)
            // default = 'default'
            default_type: default
            // 全局查id的时候是从最上层开始找还是从最下层开始找（每一层都可能有配置文件）
            // default = false
            deep_first: false
            // true=环境变量env都是全局的（全局查找），否则优先每个配置文件里查环境变量，查不到才查全局
            // default = true
            global_env: true
            // 环境变量读取顺序，默认先命令行配置，然后系统变量，然后本地配置文件配置(设置全局则不查)，最后配置文件配置，不想读哪个把哪个删了就可以
            // 命令行配置格式:
            //     -e a=b --env=a=b --env=c=d
            //     env: {a=b, c=d}
            env_orders: [args, sys, local, conf]
            env_from_args: false
            // 命令行读取方式：默认xf(xf.args),可选: buildz(buidlz.argx)
            args_type: 'xf'
            // true=类型处理函数deal都是全局的（全局查找），否则优先每个配置文件里查处理函数，查不到才查全局
            // default = true 
            global_deal: true
            // 数据的id字段名
            // default = 'id'
            data_key_id: id
            // 数据的type字段名
            // default = 'type'
            data_key_type: type
            // 数据配置参数是数组的时候，数据id的位置
            // default=[0,0]
            data_index_id: [0,1]
            // 数据配置参数是数组的时候，数据type的位置
            // default=[0,1]
            data_index_type: [0,0]
            // 处理deal的type字段名
            // default = 'type'
            deal_key_type: type
            // deal配置参数是数组的时候，数据type的位置
            // default=0
            deal_index_type: 0
        }
        """
        if loads is None:
            loads = xf.loads
        self.loads = loads
        if type(conf) in [bytes, str]:
            conf = self.loads(conf)
        self.spt = xf.g(conf, spt = ".")
        self.env_spt = xf.g(conf, env_spt = ".")
        self.default_type = xf.g(conf, default_type='default')
        self.deep_first = xf.g(conf, deep_first=False)
        self.global_env = xf.g(conf, global_env = True)
        self.global_deal = xf.g(conf, global_deal = True)
        self.data_key_id = xf.g(conf, data_key_id = 'id')
        self.data_key_type = xf.g(conf, data_key_type = 'type')
        self.data_index_id = xf.g(conf, data_index_id = [0,1])
        self.data_index_type = xf.g(conf, data_index_type = [0,0])
        self.deal_key_type = xf.g(conf, deal_key_type = 'type')
        self.deal_index_type = xf.g(conf, deal_index_type = 0)
        self.env_orders = xf.g(conf, env_orders = None)
        self.env_from_args = xf.g(conf, env_from_args = False)
        if self.env_orders is None:
            self.env_orders = ['sys', 'local', 'conf']
            if self.env_from_args:
                self.env_orders = ['args', 'sys', 'local', 'conf']
        self.env_fcs = {
            'args': self.get_env_args,
            'sys': self.get_env_sys,
            'local': self.get_env_local,
            'conf': self.get_env_conf
        }
        self.args_type = xf.g(conf, args_type = "xf")
        self._conf_id = 0
        self.conf = conf
        self.node = ConfsNode()
        self.confs = {}
        self.deals = {}
        self.envs = {}
        self.envs_args = None
        self.mark_init = False
        self.vars = {}
        self.fcs = {}
        if 'args' in self.env_orders:
            self.build_env_args()
        _id = self.add({})
        self.empty = self.confs[_id]
    def set_fc(self, key, fc):
        self.fcs[key] = fc
    def get_fc(self, key):
        if key not in self.fcs:
            return None
        return self.fcs[key]
    def var_keys(self):
        return self.vars.keys()
    def get_var(self, key, i=-1):
        if not self.has_var(key):
            return None, False
        return self.vars[key][i], True
    def push_vars(self, vars):
        if vars is not None:
            [self.push_var(key,val) for key,val in vars.items()]
        return pyz.with_out(lambda :self.pop_vars(vars))
    def pop_vars(self, vars):
        if vars is not None:
            [self.pop_var(key) for key in vars]
    def push_var(self, key, val):
        if key not in self.vars:
            self.vars[key] =  []
        self.vars[key].append(val)
    def has_var(self, key):
        if key not in self.vars:
            return False
        return len(self.vars[key])>0
    def pop_var(self, key):
        if not self.has_var(key):
            return
        self.vars[key].pop(-1)
        if len(self.vars[key])==0:
            del self.vars[key]
    def do_init(self):
        if self.mark_init:
            return
        self.mark_init = True
        for id in self.confs:
            self.confs[id].do_init()
    def get_deal_type(self, obj):
        if type(obj)==dict:
            return obj[self.deal_key_type]
        return obj[self.deal_index_type]
    def get_data_id(self, obj):
        if type(obj)==dict:
            return xf.get(obj, self.data_key_id, None)
            #return obj[self.data_key_id]
        obj = obj[self.data_index_id[0]]
        if type(obj) in [list, tuple]:
            obj = obj[self.data_index_id[1]]
        return obj
    def get_data_type(self, obj, type_first = 1, default = None):
        self.do_init()
        if type(obj)==dict:
            if self.data_key_type not in obj:
                return default
            return obj[self.data_key_type]
        obj = obj[self.data_index_type[0]]
        if type(obj) in [list, tuple]:
            return obj[self.data_index_type[1]]
        if type_first:
            return obj
        return default
    def conf_id(self):
        """
            给每个配置文件加一个id，外部不调用
        """
        id = self._conf_id
        self._conf_id+=1
        return id
    def gid(self, cid, id):
        cids = self.ids(cid)
        ids = self.ids(id)
        return self.id(cids+ids)
    def ids(self, id):
        if id is None:
            return []
        """
            data的id转id列表，外部不调用
            例: id = 'a.b.c', spt = ".", ids = ['a','b','c']
        """
        return id.split(self.spt)
    def id(self, ids):
        """
            data的id列表转id，外部不调用
            例: ids = ['a','b','c'], spt = ".", id = 'a.b.c', 
        """
        return self.spt.join(ids)
    def add_fps(self, fps):
        for fp in fps:
            self.add_fp(fp)
    def add_fp(self, fp):
        try:
            conf = self.loads(xf.fread(fp))
            return self.add(conf)
        except:
            print(f'error in loads: {fp}')
            raise
        #return self.add(conf)
    def add_decorator(self):
        conf = decorator()
        #print(f"[TESTZ] CONFS add decorator(): {conf}")
        return self.add(conf)
    def add_wrap(self):
        return self.add_decorator()
    def adds(self, confs):
        for conf in confs:
            self.add(conf)
    def add(self, conf):
        """
            {
                deals:[{build: fc_path,args: [],maps: {}}]
                envs: {id: val}
                id: default null
                namespace: default null
                datas: [{id:val, type: val, ...}]
                locals: [like datas]
            }
        """
        if type(conf) in [bytes, str]:
            conf = self.loads(conf)
        obj = Conf(conf, self)
        id = obj.namespace
        #id = xf.g1(conf, id=None, namespace=None)
        ids = self.ids(id)
        node = self.node
        for id in ids:
            if id not in node.ids:
                node.ids[id] = ConfsNode()
            node = node.ids[id]
        node.confs.append(obj)
        self.confs[obj.id] = obj
        for k in obj.deals:
            self.deals[k] = obj.deals[k]
        self.update_maps(self.envs, obj.envs)
        self.mark_init = False
        return obj.id
    def get(self, *args, **maps):
        return self.get_obj(*args, **maps)
    def remove(self, *a,**b):
        return self.get_obj(*a, **b, remove=True)
    def get_obj(self, id, sid = None, src = None, info = None, remove = False):
        """
            根据data id获取data对象，处理逻辑：根据data id查配置，根据配置的type查deal，返回deal处理过的配置
        """
        self.do_init()
        if type(id) == EncapeData:
            conf = id
        elif type(id) in [list, dict]:
            conf = EncapeData(id, conf=self.empty, confs=self, info=info, src=src, type= self.get_data_type(id))
        else:
            conf = self.get_data(id, sid, src=src, info = info)
        if conf is None:
            raise IdNotFoundError(f"confs: can't find conf of {id}")
            return None
        #print(f"[TESTZ] confs.get: {conf.data}, conf: {conf.conf}, type: {conf.type}")
        # if conf.conf is None:
        #     if remove:
        #         return None
        #     return conf()
        deal = self.get_deal(conf.type, sid)
        if deal is None:
            raise IOCError(f"confs: can't find deal of {id}, type = {conf.type}")
            return None
        #print(f"get_obj: {id}({sid}), conf: {conf}, deal: {deal}, type: {conf.type}")
        if not remove:
            obj = deal(conf)
        else:
            obj = deal.remove(conf)
        return obj
    def get_deal(self, type, sid=None):
        """
            根据type类型查处理函数deal，sid（配置文件id）不为空并且global_deal=False则先查局部
        """
        self.do_init()
        if sid is not None and not self.global_deal:
            deal = self.confs[sid].get_deal(type, False)
            if deal is not None:
                return deal
        if type in self.deals:
            return self.deals[type]
        return None
    def full_ids(self, src=None):
        sid = None
        if src is not None:
            sid = src.id
        rst = []
        for _id, conf in self.confs.items():
            rst+=conf.full_ids(sid==conf.id)
        return rst
    def get_confs(self, ids):
        """
            根据ids查所有对应的配置文件列表
        """
        node = self.node
        confs = []
        for i in range(len(ids)):
            id = ids[i]
            confs.append([node.confs,i])
            if id not in node.ids:
                break
            node = node.ids[id]
        return confs
    def get_data(self, id, sid=None, src = None, info = None):
        """
            根据id查对应的data配置
        """
        ids = self.ids(id)
        arr = self.get_confs(ids)
        if self.deep_first:
            arr.reverse()
        for confs,i in arr:
            id = self.id(ids[i:])
            for conf in confs:
                _conf = conf.get_data(id, sid==conf.id, False, src = src, info = info)
                if _conf is not None:
                    return _conf
        fc = self.get_fc(id)
        if fc is None:
            return None
        return EncapeData(fc, confs=self)

pass

class ConfsList(Base):
    def init(self, mgs):
        self.mgs = mgs
    def get(self, conf):
        _exp = None
        for mg in self.mgs:
            try:
                obj = mg.get(conf)
                return obj
            except IdNotFoundError as exp:
                _exp = exp
        raise _exp
    def push_vars(self, vars):
        [mg.push_vars(vars) for mg in self.mgs]
        return pyz.with_out(lambda :self.pop_vars(vars))
    def pop_vars(self, vars):
        [mg.pop_vars(vars) for mg in self.mgs]

pass






'''
数据封装类
val是数据（字符）值
pos是字符在文件里的位置(第几行第几列)
remain和type？
'''
class ShowItem:
    def __str__(self):
        return "<show_item val={val}, deep = {deep}, maps={maps}>".format(val = str(self.val), maps=self._maps, deep = self.deep)
    def __repr__(self):
        return str(self)
    def none(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return False
        return True
    def any(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)==v:
                return True
        return False
    def check(self, **maps):
        for k in maps:
            v = maps[k]
            if self.maps(k)!=v:
                return False
        return True
    def maps(self, key):
        if key not in self._maps:
            if key == 'type':
                return self.type
            return None
        return self._maps[key]
    def __init__(self, val, deep=1,  **maps):
        self.val= val
        self._maps = maps
        self.deep = deep

pass


#coding=utf-8
import sys
class Fetch:
    """
    命令行参数读取
    ft = Fetch("id,name,kind".split(","), {"a":"age"})
        or
    ft = Fetch(*xf.loads("[[id,name,kind],{a:age}]"))

    ft("001 zero life -a12".split(" ")) = {'id': '001', 'name': 'zero', 'kind': 'life', 'age': '12'}

    但更简单的方法是:
    xf.args("{id:'001', name: zero, kind: life, age: 12}".split(" ")) = {'id': '001', 'name': 'zero', 'kind': 'life', 'age': 12}
    python buildz.xf {id:'001', name: zero, kind: life, age: 12}
    就是对引号不太适用
    """
    def __init__(self, args = [], maps ={}):
        args = [maps[k] if k in maps else k for k in args]
        self.args = args
        self.maps = maps
    def des(self):
        cmd = " ".join(self.args)
        adds = []
        for k, val in self.maps.items():
            tmp = f"    [-{k} param] [--{val}=param]"
            adds.append(tmp)
        adds = [cmd]+adds
        rs = "\n".join(adds)
        return rs
    def check(self, args, ks):
        rst = []
        for k in ks:
            if k not in args:
                rst.append(k)
        return rst
    def __call__(self, argv = None):
        args, maps = fetch(argv)
        rst = {}
        for i in range(len(args)):
            if i >= len(self.args):
                break
            key = self.args[i]
            rst[key] = args[i]
        for key in maps:
            rst[key] = maps[key]
        for key in self.maps:
            rkey = self.maps[key]
            if key in rst:
                val = rst[key]
                rst[rkey] = val
                del rst[key]
        return rst

pass
class FetchX(Fetch):
    def __init__(self, *a, **b):
        super().__init__(a,b)

pass
def fetch(argv = None):
    r"""
    format: a b c -a 123 -b456 --c=789 +d  -x"??? ???" y z
    return: [a,b,c,y,z], {a:123,b:456,c:789,d:1,x:'??? ???'}
    """
    if argv is None:
        argv = sys.argv[1:]
    lists, maps = [],{}
    argv = [str(k).strip() for k in argv]
    argv = [k for k in argv if k!=""]
    i = 0
    while i<len(argv):
        v = argv[i]
        make_plus = 0
        if v in ["-", "--", "+"]or v[0] not in "+-":
            lists.append(v)
            i+=1
            continue
        if v[0] == "+":
            key = v[1:]
            make_plus = 1
            val = 1
        else:
            if v[1]=="-":
                kv = v[2:]
                x = kv.split("=")
                key = x[0]
                val = "=".join(x[1:])
                if len(val)==0:
                    val = 1
            else:
                key = v[1]
                if len(v)>2:
                    val = v[2:]
                else:
                    if i+1>=len(argv):
                        val = 1
                    else:
                        val = argv[i+1]
                        i+=1
        if make_plus:
            keys = key.split(",")
        else:
            keys = [key]
        for key in keys:
            if key not in maps:
                maps[key] = []
            maps[key].append(val)
        i+=1
    for k in maps:
        v = maps[k]
        if len(v)==1:
            maps[k] = v[0]
    return lists, maps

pass

def get(maps, keys, default=None):
    if type(keys) not in [list, tuple]:
        keys = [keys]
    for key in keys:
        if key in maps:
            return maps[key]
    return default

pass
        




#coding=utf-8

class Stack:
    def __getitem__(self, i):
        if i>=0:
            n = self.base+i
        else:
            n = self.last+i
        if n<self.base or n>=self.last:
            raise IndexError("index out of range")
        if n<0:
            raise Exception("up N<0")
            n+=self.size
        elif n>=self.size:
            raise Exception("up N>sz")
            n-=self.size
        return self.stack[n]
        if i == 0:
            return self.stack[self.base]
        elif i==-1:
            n = self.last-1
            if n<0:
                n+=self.size
            return self.stack[n]
        else:
            raise Exception("unsupport")
    def __init__(self, size):
        self.stack = [None]*size
        self.base = 0
        self.last = 0
        self.size = size
    def pop(self, i):
        if i==0:
            obj = self.stack[self.base]
            self.base+=1
            if self.base>self.last:
                raise Exception("unsupport X1")
            if self.base>=self.size:
                raise Exception("unsupport B")
                self.base -=self.size
        elif i==-1:
            self.last -=1 
            if self.base>self.last:
                raise Exception("unsupport X1")
            if self.last<0:
                raise Exception("unsupport A")
                self.last+=self.size
            obj = self.stack[self.last]
        else:
            raise Exception("unsupport")
        return obj
    def __len__(self):
        n = self.last - self.base
        if n<0:
            raise Exception("unsupport N1")
            n+=self.size
        elif n>=self.size:
            raise Exception("unsupport N2")
            n -= self.size
        return n
    def append(self, obj):
        self.stack[self.last]=obj
        self.last+=1
        if self.last>=self.size:
            raise Exception("unsupport C")
            self.last-=self.size

pass


from .. import base
from .. import item

class PrevNextDeal(base.BaseDeal):
    def labels(self):
        return ['']
    #def types(self):
    #    return [""]
    """
        读取下一个字符放缓存里，应放最低优先级
    """
    def deal(self, buffer, arr, mg):
        c = buffer.read_cache(1)
        if len(c)==0:
            rm = buffer.full().strip()
            buffer.clean()
            if len(rm)==0:
                return False
            obj = item.Item(rm, type = 'str', is_val = 0)
            arr.append(obj)
            # TODO return True?
            return False
        return True

pass


#
from .ioc.decorator import decorator
from .ioc_deal import init
#print(f"decorator.fcs: {decorator.fcs}")
locals().update(decorator.fcs)
from buildz import fz
import os
def pkg(fp):
    r = fp.rfind(".")
    if r>0:
        fp = fp[:r]
    fp = fp.replace("\\", "/")
    fp = fp.replace("//", "/")
    fp = fp.replace("/", ".")
    while len(fp)>0 and fp[0]==".":
        fp = fp[1:]
    __import__(fp)

pass
def imports(dp, pts, pfx=""):
    """
    find filepath by dirpath(dp) and pattern(pts), 
    import:
    filepath = ".".join(filepath.split(".")[:-1])
    filepath = filepath[len(dp):]
    package = pfx+"." if pfx!="" else "")+(filepath.replace("/", "."))
    """
    dp = os.path.abspath(dp)
    fps = fz.search(dp, pts)
    fps = [pfx+"/"+fp[len(dp):] for fp in fps]
    for fp in fps:
        pkg(fp)

pass


from .. import base
from .. import item
from .. import exp
from . import lr
class MapDeal(lr.LRDeal):
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def init(self, left, right):
        super().init(left, right, "map")
    def types(self):
        return ['list']
    def build(self, obj):
        #if self.check_right(obj):
        #    return None
        val = obj.val
        if len(val)==0:
            obj.val = {}
            obj.is_val = 1
            return obj
        if len(val)%3!=0:
            return None
        opt = val[1]
        if opt.type!='kv':
            return None
        return self.build_arr(val)
    def build_arr(self, arr):
        rst = {}
        if len(arr)%3!=0:
            raise Exception(f"u f in map: {arr}")
        for i in range(0, len(arr), 3):
            k = arr[i]
            v = arr[i+2]
            opt = arr[i+1]
            if opt.type != "kv":
                raise Exception(f"u f opt in map: {opt}")
            rst[k.val] = v.val
        return item.Item(rst, type='map', is_val = 1)

pass



class BufferInput:
    def __init__(self, buf):
        self.buf = buf
    def __call__(self, size=1):
        s = self.buf[:size]
        self.buf = self.buf[size:]
        return s

pass

class Buffer:
    def read(self, size = 1, pop = 0):
        if self.s_read is None:
            self.s_read = self.input(1)
        l = len(self.s_read)
        if l<size:
            self.s_read += self.input(size-l)
        rst = self.s_read[:size]
        if pop:
            self.s_read = self.s_read[size:]
        if self.buffer is None:
            self.buffer = self.s_read[:0]
        return rst
    def pop_read(self, size=1):
        self.s_read = self.s_read[size:]
    def init(self):
        self.buffer = None
        self.s_read = None
    def __init__(self, input):
        self.input = input
        self.init()
    def add(self, arr):
        if self.buffer is None:
            self.buffer = arr
        else:
            self.buffer += arr
    def size(self):
        if self.buffer is None:
            return 0
        return len(self.buffer)
    def full(self, size = 0, right = 1):
        if right:
            return self.rfull(size)
        return self.lfull(size)
    def clean(self):
        self.buffer = self.buffer[:0]
    def get(self, size=1, right=1):
        if right:
            return self.rget(size)
        return self.lget(size)
    def pop(self, size=1, right=1):
        if right:
            return self.rpop(size)
        return self.lpop(size)
    def lfull(self, size = 0):
        if self.buffer is None:
            return self.buffer
        if size == 0:
            return self.buffer
        return self.buffer[size:]
    def rfull(self, size = 0):
        if self.buffer is None:
            return self.buffer
        if size == 0:
            return self.buffer
        return self.buffer[:-size]
    def lget(self, size=1):
        if self.buffer is None:
            return self.buffer
        return self.buffer[:size]
    def rget(self, size=1):
        if self.buffer is None:
            return self.buffer
        return self.buffer[-size:]
    def lpop(self, size = 1):
        if self.buffer is None:
            return self.buffer
        self.buffer = self.buffer[size:]
    def rpop(self, size=1):
        if self.buffer is None:
            return self.buffer
        self.buffer = self.buffer[:-size]

pass

class StrBuffer:
    def read(self, size = 1, pop = 0):
        s = self.str[self.read_base:self.read_base+size]
        if pop:
            self.read_base+=size
        return s
    def pop_read(self, size=1):
        self.read_base+=size
    def init(self):
        self.buffer_base = 0
        self.buffer_size = 0
        self.read_base = 0
        self.read_size = 0
        #self.buffer = None
        #self.s_read = None
    def __init__(self, s):
        self.str = s
        self.init()
    def add(self, arr):
        # x = self.str[self.buffer_base+self.buffer_size:self.buffer_base+self.buffer_size+len(arr)]
        # if x!=arr:
        #     print(f"[ERROR] x:({x}), arr:({arr}), bb: {self.buffer_base}, bs:{self.buffer_size}, rb: {self.read_base}")
        #     print("[["+self.str[self.buffer_base:self.read_base+10]+"]]")
        #     raise Exception("")
        self.buffer_size+=len(arr)
    def size(self):
        return self.buffer_size
    def full(self):
        return self.str[self.buffer_base:self.buffer_base+self.buffer_size]
    def clean(self):
        self.buffer_base = self.read_base
        self.buffer_size = 0

pass



#
from ..tools import *
from buildz.ioc import wrap
import os
from buildz.db.dv import build
@wrap.obj(id="dbs")
@wrap.obj_args("ref, cache", "ref, log")
class Dbs(Base):
    def init(self, cache, log):
        self.cache = cache
        self.log = log
        self.dbs = {}
    def call(self, maps, fp):
        confs = xf.g(maps, dbs={})
        for key,conf in confs.items():
            url,user,pwd,dv = xf.g(conf, url=None, user=None, pwd=None, device=key)
            dv = build(dv, [url, user, pwd], val)
            self.dbs[key] = dv
        self.cache.set_mem("dbs", self.dbs)
        return True

pass


